"""Pylons environment configuration"""
import os

from ConfigParser import RawConfigParser
from jinja2 import ChoiceLoader, Environment, FileSystemLoader
import os.path
import pylons
from pylons.configuration import PylonsConfig
from sqlalchemy import create_engine

from sloecode.bzr.factory import BazaarRepositoryFactory
import sloecode.lib.app_globals as app_globals
import sloecode.lib.helpers
from sloecode.config.routing import make_map
from sloecode.model import init_model


def load_environment(global_conf, app_conf):
    """Configure the Pylons environment via the ``pylons.config``
    object
    """
    config = PylonsConfig()

    # Pylons paths
    root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    paths = dict(root=root,
                 controllers=os.path.join(root, 'controllers'),
                 static_files=os.path.join(root, 'public'),
                 templates=[os.path.join(root, 'templates')])

    # Initialize config with the basic options
    config.init_app(global_conf, app_conf, package='sloecode', paths=paths)

    config['routes.map'] = make_map(config)
    config['pylons.app_globals'] = app_globals.Globals(config)
    config['pylons.h'] = sloecode.lib.helpers

    # Setup cache object as early as possible
    pylons.cache._push_object(config['pylons.app_globals'].cache)

    # Create the Jinja2 Environment
    config['pylons.app_globals'].jinja2_env = Environment(
        extensions=['jinja2.ext.do'],
        loader=ChoiceLoader(
            [FileSystemLoader(path) for path in paths['templates']]))
    # Jinja2's unable to request c's attributes without strict_c
    config['pylons.strict_tmpl_context'] = True

    # Setup the SQLAlchemy database engine
    # do this by creating the engine url from components:
    db_config = RawConfigParser()
    db_config.read(config['db_config_file'])
    
    dialect = db_config.get('database', 'dbtype')
    # translate from dbconfig-common terminology
    if dialect == 'sqlite3':
        dialect = 'sqlite'
    username = db_config.get('database', 'dbuser')
    password = db_config.get('database', 'dbpass')
    schema_name = db_config.get('database', 'dbname')
    db_server = db_config.get('database', 'dbserver')
    db_port = db_config.get('database', 'dbport')
    db_path = db_config.get('database', 'dbpath')
    
    # we support sqlite:// and mysql://
    engine_url = dialect + '://'
    # sqlite needs 4 '/''s, and needs special path options:
    if dialect == 'sqlite':
        engine_url += '/' + os.path.join(db_path, schema_name)
    else:
        engine_url += "%s:%s@%s/%s" % (username, password, db_server, schema_name)
    
    engine = create_engine(engine_url)
    init_model(engine)

    # create the Bazaar factory:
    config['bzr.factory'] = BazaarRepositoryFactory(
        global_conf['bzr_person_root'],
        global_conf['bzr_project_root'],
        global_conf['bzr_deleted_person_root'],
        global_conf['bzr_deleted_project_root'],
        )
        
    # This is a bit of a nasty hack: In "developer mode", we explicitly call:
    #
    # `paster setup-app /path/to/config.ini`
    #
    # Which creates all the database tables, and also created the admin user and
    # creates the repository for the admin user - this works fine.
    #
    # However, in release mode, the database schema is created by dbconfig-common
    # and upgraded from one version to the next by the same. We never get a chance
    # to create the Bazaar repository for the admin user, which results in crashes.
    #
    # The following code checks to see if the repository has been created for the
    # admin user, and creates it if it hasn't:
    from bzrlib.errors import NotBranchError

    try:
        repo = config['bzr.factory'].get_repository_for_person('admin')
    except NotBranchError:
        config['bzr.factory'].create_shared_repository_for_person('admin')


    return config
