"""Project model.
"""

import formencode
import re

from sqlalchemy import Column
from sqlalchemy.orm import relationship
from sqlalchemy.types import Integer, String, Text


from sloecode.model.meta import Base, QueryMixin


class Project(Base, QueryMixin):
    """Project model.

    Projects have project names and descriptions. Will probably contain more
    in the future.
    """
    __tablename__ = "project"

    id = Column(Integer, primary_key=True)
    name = Column(String(24), unique=True)
    displayname = Column(String(32))
    description = Column(Text())

    users = relationship('Membership', backref='project',
        cascade="all, delete-orphan")


class UniqueProject(formencode.FancyValidator):
    "A simple formencode validator that checks Project names are unique."

    project_regex = re.compile(r'^[a-z0-9][a-z0-9_.-]{2,23}$')

    def _to_python(self, value, state):
        """Validate form-data.
        """
        value = value.strip()
        if len(Project.get(name=value)) > 0:
            raise formencode.Invalid(
                'A project with that name already exists.', value, state)

        if not self.project_regex.match(value):
            raise formencode.Invalid(
                'A project name must be lower case letters, numbers, or _-., and be 3-24 characters long.',
                value, state)

        return value


class ProjectSchema(formencode.Schema):
    """A formencode schema for the Project object."""

    allow_extra_fields = True
    filter_extra_fields = True

    name = UniqueProject(not_empty=True)
    displayname = formencode.validators.String(not_empty=True, max=32)
    description = formencode.validators.String(not_empty=False)
